﻿//Copyright Warren Clayton Harding 2015
//Released to the public domain
//under the Ms-pl
//Please leave this copyright notice intact
using System;

namespace Servo
{
    public class PositionServo
    {
        public Double maxForce = 100000; //should be set to the maximum force output of your motor
        public Double accelerationSlope = 100000; //indicates how rapidly acceleration increases further from the target, increase to increase overall speed
        public Double lastCalculatedMassLikeResistance = 1; //mass is not always known and there will be other factors like friction, so this is calculated by the algorithm
        public Double lastVelocity = 0; //calculated by the algorithm
        public Double lastPositionFromZero = 0; //calculated by the algorithm
        public Double lastForce = 1; //calculated by the algorithm

        //the main function for you to call.
        //Returns a force that is the force required of the motor,
        //convert this to a power if you are using a real motor and send it to the motor
        //physics engines often have an AddForce function on the rigidbodies that you
        //can send this to if you are building a simulation
        public Double Force(Double position, Double targetPosition, Double elapsedTime)
        {
            Double positionFromZero = position - targetPosition; //target at zero coordinates
            Double velocity = (positionFromZero - lastPositionFromZero) / elapsedTime;
            Double observedAcceleration = (velocity - lastVelocity) / elapsedTime;
            Double calculatedMassLikeResistance = lastForce / observedAcceleration;
            if (Double.IsNaN(calculatedMassLikeResistance))
            {
                calculatedMassLikeResistance = lastCalculatedMassLikeResistance;
            }

            Double acceleration = Acceleration(positionFromZero, velocity, accelerationSlope);
            Double force = calculatedMassLikeResistance * acceleration;

            if (Math.Abs(force) > maxForce)
            {
                force = maxForce * Math.Sign(force);
            }
            lastPositionFromZero = positionFromZero;
            lastVelocity = velocity;
            lastForce = force;
            lastCalculatedMassLikeResistance = calculatedMassLikeResistance;
            return force;
        }

        //in target at zero coordinates, you probably won't call this directly
        public static Double Acceleration(Double position, Double velocity, Double accelerationSlope)
        {
            //equations of constant slope of acceleration
            //a=b/t
            //v=b*t^2
            //x=b*t^3/3

            //constant acceleration
            //Double timeForPerfectDeceleration = Math.Pow(2d * Math.Abs(position) / accelerationSlope, 1d / 2d);
            //Double velocityForPerfectDeceleration = accelerationSlope * timeForPerfectDeceleration / 2d;
            //Double accelerationForPerfectDeceleration = accelerationSlope;

            //constant slope of acceleration
            //Double timeForPerfectDeceleration = Math.Pow(3d * Math.Abs(position) / accelerationSlope, 1d / 3d);
            //Double velocityForPerfectDeceleration = accelerationSlope * timeForPerfectDeceleration * timeForPerfectDeceleration / 2d;
            //Double accelerationForPerfectDeceleration = accelerationSlope * timeForPerfectDeceleration;

            //constant slope of slope of acceleration
            Double timeForPerfectDeceleration = Math.Pow(4d * Math.Abs(position) / accelerationSlope, 1d / 4d);
            Double velocityForPerfectDeceleration = accelerationSlope * Math.Pow(timeForPerfectDeceleration, 3d) / 3d;
            Double accelerationForPerfectDeceleration = accelerationSlope * Math.Pow(timeForPerfectDeceleration, 2d) / 2d;

            ///...and so on....
            //Double timeForPerfectDeceleration = Math.Pow(6d * Math.Abs(position) / accelerationSlope, 1d / 6d);
            //Double velocityForPerfectDeceleration = accelerationSlope * Math.Pow(timeForPerfectDeceleration, 5d) / 5d;
            //Double accelerationForPerfectDeceleration = accelerationSlope * Math.Pow(timeForPerfectDeceleration, 4d) / 4d;

            velocityForPerfectDeceleration = velocityForPerfectDeceleration * -Math.Sign(position);
            if (position < 0)
            {
                if (velocity <= velocityForPerfectDeceleration)
                {
                    return accelerationForPerfectDeceleration;
                }
                else
                {
                    return -accelerationForPerfectDeceleration;
                }
            }
            else
            {
                if (velocity >= velocityForPerfectDeceleration)
                {
                    return -accelerationForPerfectDeceleration;
                }
                else
                {
                    return accelerationForPerfectDeceleration;
                }
            }
        }
    }
}
