﻿//Copyright Warren Clayton Harding 2015
//Released to the public domain
//under the Ms-pl
//Please leave this copyright notice intact
using System;

namespace Servo
{
    //The main RotationServo class
    //It relies heavily on inheriting from PositionServo so read the comments there
    //Use the main Force function here to get a force, you can enter angles however you want,
    //the class won't get confused and will send the motor on its way by the shortest rotation
    //enter angles in degrees
    public class RotationServo : PositionServo
    {
        public new Double Force(Double inCurrentAngle, Double inTargetAngle, Double elapsedTime)
        {
            Double desiredAngleChange;
            desiredAngleChange = AngleFromAngle1ToAngle2Degrees(inCurrentAngle, inTargetAngle);
            return base.Force(-desiredAngleChange, 0, elapsedTime);
        }

        static Double AngleFromAngle1ToAngle2Degrees(Double angle1, Double angle2)
        {
            angle1 = WrapAngleDegrees(angle1);
            angle2 = WrapAngleDegrees(angle2);
            if (angle2 - angle1 < 180 && angle2 > angle1)
            {
                return angle2 - angle1;
            }
            else if (angle2 - angle1 > 180 && angle2 > angle1)
            {
                return -(360 - (angle2 - angle1));
            }
            else if (angle1 - angle2 < 180 && angle1 > angle2)
            {
                return -(angle1 - angle2);
            }
            else if (angle1 - angle2 > 180 && angle1 > angle2)
            {
                return 360 - (angle1 - angle2);
            }
            else if (angle1 - angle2 == 180)
            {
                return 180;
            }
            else if (angle2 - angle1 == 180)
            {
                return 180;
            }
            return 0;
        }

        static Double WrapAngleDegrees(Double angle)
        {
            angle = (Double)Math.IEEERemainder(angle, 360);
            if (angle <= 0)
            {
                angle += 360;
            }
            else
            {
                if (angle > 360)
                {
                    angle -= 360;
                }
            }
            return angle;
        }
    }
}
