﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Servo;
namespace ServoTest
{
    /// <summary>
    /// Interaction logic for PositionTest.xaml
    /// </summary>
    public partial class PositionTest : Window
    {
        Ellipse targetEllipse;
        Ellipse shapeEllipse;
        PositionServo positionServo;
        Double targetPosition;
        Double mass;
        Double acceleration;
        Double velocity;
        Double position;
        Boolean isBusy;

        public void SetDefaults()
        {
            positionServo = new PositionServo();
            targetPosition = Convert.ToDouble(txtTargetPosition.Text);
            mass = .1f;
            acceleration = 0;
            velocity = 0;
            position = 100;
            isBusy = false;
            Canvas.SetLeft(targetEllipse, targetPosition);
            positionServo.maxForce = Convert.ToDouble(txtMaxForce.Text);
            positionServo.accelerationSlope = Convert.ToDouble(txtAccelerationSlope.Text);
        }

        public PositionTest()
        {
            InitializeComponent();

            targetEllipse = new Ellipse();
            targetEllipse.Stroke = System.Windows.Media.Brushes.Black;
            targetEllipse.Fill = System.Windows.Media.Brushes.DarkRed;
            targetEllipse.Width = 50;
            targetEllipse.Height = 50;
            canvas1.Children.Add(targetEllipse);
            Canvas.SetLeft(targetEllipse, targetPosition);

            shapeEllipse = new Ellipse();
            shapeEllipse.Stroke = System.Windows.Media.Brushes.Black;
            shapeEllipse.Fill = System.Windows.Media.Brushes.DarkBlue;
            shapeEllipse.Width = 50;
            shapeEllipse.Height = 50;
            canvas1.Children.Add(shapeEllipse);
            Canvas.SetLeft(shapeEllipse, position);

            if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this) == false)
            {
                CompositionTarget.Rendering += CompositionTarget_Rendering;
            }

            txtTargetPosition.KeyUp += TxtTargetPosition_KeyUp;

        }

        private void TxtTargetPosition_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                targetPosition = Convert.ToDouble(txtTargetPosition.Text);
                Canvas.SetLeft(targetEllipse, targetPosition);
            }
        }

        Boolean isOn = false;
        void CompositionTarget_Rendering(object sender, EventArgs e)
        {
            if (isOn)
            {
                Double elapsed = 1d / 60d;
                if (elapsed == 0)
                {
                    return;
                }
                Double force = positionServo.Force(position, targetPosition, elapsed);
                acceleration = force / mass;
                velocity += acceleration * elapsed;
                position += velocity * elapsed;

                if (isBusy == true)
                {
                    return;
                }
                isBusy = true;

                this.Dispatcher.Invoke(() => Canvas.SetLeft(shapeEllipse, position));
                isBusy = false;
            }
        }

        private void btnStart_Click_1(object sender, RoutedEventArgs e)
        {
            if (isOn)
            {
                isOn = false;
            }
            SetDefaults();
            isOn = true;
        }

        private void btnStop_Click_1(object sender, RoutedEventArgs e)
        {
            isOn = false;
        }
    }
}
