﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Servo;
namespace ServoTest
{
    /// <summary>
    /// Interaction logic for RotationTest.xaml
    /// </summary>
    public partial class RotationTest : Window
    {
        Ellipse targetEllipse;
        Ellipse shapeEllipse;
        //InfiniteLoop infiniteLoop;
        RotationServo rotationServo;
        Double targetAngle;
        Double mass;
        Double acceleration;
        Double velocity;
        Double position;
        Boolean isBusy;
        Boolean isOn;

        public void SetDefaults()
        {
            rotationServo = new RotationServo();
            mass = 1;
            acceleration = 0;
            velocity = 0;
            position = 0;
            isBusy = false;
            Canvas.SetLeft(targetEllipse, 0);
            targetAngle = Convert.ToSingle(this.txtTarget.Text);
            position = Convert.ToSingle(this.txtEffector.Text);
            isOn = false;
        }

        public RotationTest()
        {
            InitializeComponent();

            SetTargetEllipse();

            shapeEllipse = new Ellipse();
            shapeEllipse.Stroke = System.Windows.Media.Brushes.Black;
            shapeEllipse.Fill = System.Windows.Media.Brushes.DarkBlue;
            shapeEllipse.Width = 50;
            shapeEllipse.Height = 100;
            canvas1.Children.Add(shapeEllipse);
            SetEllipseTransform();

            this.Closing += MainWindow_Closing;

            if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this) == false)
            {
                CompositionTarget.Rendering += CompositionTarget_Rendering;
            }

            this.txtTarget.KeyUp += txtTarget_KeyUp;
            this.txtEffector.KeyUp += txtEffector_KeyUp;
        }

        private void SetTargetEllipse()
        {
            targetEllipse = new Ellipse();
            targetEllipse.Stroke = System.Windows.Media.Brushes.Black;
            targetEllipse.Fill = System.Windows.Media.Brushes.DarkRed;
            targetEllipse.Width = 50;
            targetEllipse.Height = 100;
            canvas1.Children.Add(targetEllipse);
            SetTargetEllipseTransform();
        }

        void txtEffector_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                SetEllipseTransform();
            }
        }

        private void SetEllipseTransform()
        {
            TransformGroup transform = new TransformGroup();
            transform.Children.Add(new RotateTransform(Convert.ToDouble(this.txtEffector.Text), 25, 0));
            transform.Children.Add(new TranslateTransform(100, 100));
            shapeEllipse.RenderTransform = transform;
        }

        void txtTarget_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                SetTargetEllipseTransform();
            }
        }

        private void SetTargetEllipseTransform()
        {
            TransformGroup transform = new TransformGroup();
            transform.Children.Add(new RotateTransform(Convert.ToDouble(this.txtTarget.Text), 25, 0));
            transform.Children.Add(new TranslateTransform(100, 100));
            targetEllipse.RenderTransform = transform;
        }

        void CompositionTarget_Rendering(object sender, EventArgs e)
        {
            if (isOn)
            {
                Double elapsed=1d/60d;
                if (elapsed == 0)
                {
                    return;
                }
                Double force = rotationServo.Force(position, targetAngle, elapsed);
                acceleration = force / mass;
                velocity += acceleration * elapsed;
                position += velocity * elapsed;

                if (isBusy == true)
                {
                    return;
                }
                isBusy = true;

                TransformGroup transform = new TransformGroup();
                transform.Children.Add(new RotateTransform(position, 25, 0));
                transform.Children.Add(new TranslateTransform(100, 100));
                shapeEllipse.RenderTransform = transform;
                isBusy = false;
            }
        }

        void MainWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            System.Threading.Thread.Sleep(100);
        }

        private void btnStart_Click_1(object sender, RoutedEventArgs e)
        {
            SetEllipseTransform();
            SetTargetEllipseTransform();
            if (isOn)
            {
                isOn = false;
            }
            SetDefaults();
            isOn = true;
        }

        private void btnStop_Click_1(object sender, RoutedEventArgs e)
        {
            isOn = false;
            SetEllipseTransform();
        }
    }
}
